function [ret,phan]=Aqual2_findSphereContrastSpheres(phan,theory,verbose)
if ~exist('verbose','var')
    verbose=0;
end
ret=0;
% assume we have already found as many spheres as possible and have a
% consistent coordinate voxel coordinate systems in which
% the rest of the spheres have determined the best expected positions
% for voxels and stored them in voxTheoryPositions (this is likely to be
% after a linear fit)
% we will not go a position search for these.  We will carve out a ball of
% the appropriate radius+2mm and threshold to get properties
sphereRadius=15;
searchSize=sphereRadius+4;
targetVolume=4/3*pi*(sphereRadius^3) /phan.voxelVolume;
for i=1:4
    target=theory.CNRSphereIndex(i);
    % N.B. We no longer use i for an index; target should be used

    prop=struct('Centroid',...
        [phan.voxTheoryPositions(target,2),phan.voxTheoryPositions(target,1),phan.voxTheoryPositions(target,3)]);

    [regionImg,subIdx]=getClippedSubRegion(phan,searchSize,prop);
    if verbose>10
        tmpmax=max(regionImg(:));
        tmpsz=size(regionImg);
        figure
        imshow(regionImg(:,:,floor(tmpsz(3)/2)),[0,tmpmax]);
    end
    thresh=OtsuThreshold(regionImg);
    timg=regionImg>thresh;
    limg=bwlabeln(timg);
    props=regionprops(limg,'basic');
    if length(props)>0
        defect=([props.Area] - targetVolume)./targetVolume;
        dlow=defect>-0.25;
        dhigh=defect<0.1;
        dall=dlow.*dhigh;
        inWindow=find(dall);
        if length(inWindow)>1
            disp('bad bad bug too many contrast spheres -- should never happen');
            disp(sprintf('setting contrast properties for sphere %d to zero!',target));
            phan.intensityMean(target)=0;
            phan.intensityStd(target)=0;
        elseif length(inWindow)==0
            disp(sprintf('cannot region large enough to find contrast sphere %d',target));
            phan.intensityMean(target)=0;
            phan.intensityStd(target)=0;
        else % implied length==1
            which=find(limg(:)==inWindow);
            phan.intensityMean(target)=mean(regionImg(which));
            phan.intensityStd(target)=std(regionImg(which));
            nhood=ones(3,3,3);
            se=strel(nhood);
            clear nhood;
            tmpMask=imdilate(limg,se);
            tmpMask=imdilate(tmpMask,se);
            tmpbkgd=imdilate(tmpMask,se);
            tmpbkgd=imdilate(tmpbkgd,se);
            tmpbkgd=tmpbkgd-tmpMask;
            phan.backgroundIntensityMedian(target)=median(regionImg(find(tmpbkgd)));

            % now for more detailed background analysis we do the
            % following;
            % 1) take everything below thresh and apply Otsu's method to it
            % (call it threshWP)
            % 2) that should have separated water and plastic
            % 3) get stats for pixels above and below threshWP;
            % this all assumes that intensities vary at most linearly in
            % space across our image and (if linearly) that the contrast
            % sphere is in the middle of the regionImg;
            tmpRegion=regionImg(regionImg<thresh);
            threshWP=OtsuThreshold(tmpRegion,0);
            % threshWP is a first guess
            % but it's not uncommon for the spectra to be munged up with
            % the water peak piled up near zero.  If that happens then the
            % optimzl OtsuThreshold will separate water+plastic from
            % partial volumed voxels at the edges of the contrast sphere.
            % To rectify the situation we have to apply a little logic
            % a) A signature of pileup is that the peak in the histogram
            % of voxels will be at zero
            % b) if we observe that, then start walking up from zero bin
            % until some local minimum is found.
            [nh,xh]=hist(tmpRegion,200);
            ih=1;
            while nh(ih+1)<nh(ih)
                if xh(ih) > threshWP
                    break;
                end
                ih=ih+1;
                % defend against monotonically decreasing distributions
                if ih>=length(nh)
                    warning('problems segmenting water and plastic-- this message should logically be unreachable');
                    break;
                end
            end

            [mh,mxh]=max(nh(ih:end));
            mxh=mxh+ih-1;
        
            maxX=xh(mxh);
            if(threshWP > maxX)

                disp(['Otsu threshold seems bogus (expect to be below' ...
                    ' water peak, but it is not)']);
                tmpRegion2=tmpRegion(tmpRegion<maxX);
                try
                    threshWP=OtsuThreshold(tmpRegion2,0);
                catch
                disp('Otsu threshold failed ---');
                waterMean=0;
                waterSigma=0;
                plasticMean=0;
                plasticSigma=0;
                phan.waterPeak(target)=waterMean;
                phan.waterWidth(target)=waterSigma;
                phan.plasticPeak(target)=plasticMean;
                phan.plasticWidth(target)=plasticSigma;

                break;
                end
            end
            water=tmpRegion(tmpRegion>threshWP);
            water=water(water>0);
            plastic=tmpRegion(tmpRegion<threshWP);
            plastic=plastic(plastic>0);
            % clip out 0's since those are bound to be distributed ...
            %	weirdly -- they should come from Gibb' undershoot
            %	being clipped
            % if there are less than 30 voxels in a class we don't even try to fit
            if length(water(:))>30
                try
                    [waterMean,waterSigma]=Aqual2_windowFit(water,-2,2,verbose);
                catch
                    waterMean=0;
                    waterSigma=0;
                end
            else
                waterMean=0;
                waterSigma=0;
            end
            if length(plastic(:))>30
                try
                    [plasticMean,plasticSigma]=Aqual2_windowFit(plastic,-2,2,verbose);
                catch
                    plasticMean=0;
                    plasticSigma=0;
                end
            else
                plasticMean=0;
                plasticSigma=0;
            end
            string=sprintf('water:  %f %f  plastic:  %f %f\n',waterMean,waterSigma,plasticMean,plasticSigma);
            disp(string);
            phan.waterPeak(target)=waterMean;
            phan.waterWidth(target)=waterSigma;
            phan.plasticPeak(target)=plasticMean;
            phan.plasticWidth(target)=plasticSigma;

        end
    else
        phan.intensityMean(target)=0;
        phan.intensityStd(target)=0;
    end
end
return;
