function avw = ConvertDICOMToAVW ( I, H, TimeSeries )
% avw = ConvertDICOMToAVW ( I, H, TimeSeries=0 )
% From a series of DICOM images, convert to AVW format, optionally
% converting to a time series
  if ~exist ( 'TimeSeries', 'var' )
    TimeSeries = 0;
  end
  avw = struct ( 'DataType', 'AVW_SIGNED_SHORT' );
  
  % Required fields...
  % 'DataType', 'Width', 'Height', 'Depth', 'NumVols', 'Endian',
  % 'ColormapSize'

  % Fill in DICOM information first, and let the fields below
  % overwrite
  Names = fieldnames ( H(1) );
  for ii = 1:length(Names)
    Name = char ( Names(ii) ); 
    if isstruct ( getfield ( H(1), Name ) )
      % Continue for now...
      continue
    end
    if isempty ( strfind ( Name, 'Private' ) ) & isempty ( strfind ( Name, 'Filename' ) )
      if ischar ( getfield ( H(1), Name ) ) | isnumeric ( getfield ( H(1), Name ) )
        avw = setfield ( avw, Name, getfield ( H(1), Name ) );
      end
    end
  end
  
  % Remember that Analyze uses row major format, so these numbers
  % will not jive with the volume size.
  avw.Width = size ( I, 2 );
  avw.Height = size ( I, 1 );
  avw.Depth = size ( I, 3 );
  avw.NumVols = 1;
  avw.Endian = 'Little';
  avw.ColormapSize = 0;

  % Internal information
  O = H(1).ImageOrientationPatient;
  avw.RelativeImageOrientationPatient = sprintf ( '%f\\%f\\%f\\%f\\%f\\%f', O(1), O(2), O(3), O(4), O(5), O(6) );
  O = H(1).ImagePositionPatient;
  avw.Origin = sprintf ( '%f\\%f\\%f', O(1), O(2), O(3) );
  
  % Voxel size, DICOM must use SpacingBetweenSlices...
  avw.VoxelWidth = H(1).PixelSpacing(1);
  avw.VoxelHeight = H(1).PixelSpacing(2);
  
  Z = sqrt ( sum ( ( H(2).ImagePositionPatient - H(1).ImagePositionPatient ).^2) );
  avw.VoxelDepth = Z;
  
  Header = H(1);
  
  RowOrientation = Header.ImageOrientationPatient(1:3);
  ColOrientation = Header.ImageOrientationPatient(4:6);
  SliceOrientation = cross ( RowOrientation, ColOrientation );

  % By default, assume transverse
  Foo = [0 0 0];
  Foo(1) = abs ( dot ( SliceOrientation, [1 0 0] ) );
  Foo(2) = abs ( dot ( SliceOrientation, [0 1 0] ) );
  Foo(3) = abs ( dot ( SliceOrientation, [0 0 1] ) );
  
  idx = find ( Foo == max(Foo) );
  switch idx(1)
   case 1
    % Sagittal
    avw.Orientation = 'Sagittal';
    avw.AnalyzeOrient = 2;
   case 2
    % Coronal
    avw.Orientation = 'Coronal';
    avw.AnalyzeOrient = 1;
   case 3
    % Axial / Transverse
    avw.Orientation = 'Transverse';
    avw.AnalyzeOrient = 0; % Transverse/axial
  end

  % Others?
  % Re-order the slices of the image!!!
  % Analyze is expecting the opposite order from what DICOM uses...
  sz = size ( I, 3 );
  if length(size(I)) <= 3
    avw.Image = I(:,:,sz:-1:1);
  end
  if length(size(I)) == 4
    avw.Image = I(:,:,sz:-1:1,:);
  end

  if TimeSeries
    % Figure out the time series
    T = avw.AcquisitionTime;
    TimeOffset = numeric(T(1:2)) * 3600 + numeric(T(3:4)) * 60 + numeric(T(5:6));

    c = unique ( [H.SliceLocation] );
    avw.Image = I;
    NumberOfLocations = length(c);
    avw = ReorderTimeSeriesSlices ( avw, NumberOfLocations, 1 );
      
    % Get Series time...
    if isfield ( H(1), 'TriggerTime' )
      t = [H.SliceLocation; H.TriggerTime]';
      t(:,2) = t(:,2) / 1000 + TimeOffset;
      [y,Idx] = sortrows ( t );
      
      NumberOfTimePoints = size(avw.Image,4);
      avw.SeriesTime = y(1:NumberOfTimePoints,2);
    else
      avw.SeriesTime = TimeOffset;
    end
  end